# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _, ugettext
from django.utils.encoding import python_2_unicode_compatible, force_text

from cms.models import CMSPlugin


class CategoryQuerySet(models.QuerySet):
    def enabled(self):
        return self.filter(enabled=True)


@python_2_unicode_compatible
class Category(models.Model):
    objects = CategoryQuerySet.as_manager()

    name = models.CharField(
        verbose_name=_('Name'),
        max_length=500,
    )
    order = models.IntegerField(
        verbose_name=_('Order'),
        default=0,
    )
    enabled = models.BooleanField(
        verbose_name=_('Enabled'),
        default=True,
    )

    def __repr__(self):
        return '{}: {}'.format(self.id, self.name)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')
        ordering = [
            '-order',
        ]


class ReferenceQuerySet(models.QuerySet):
    def enabled(self):
        return self.filter(enabled=True)


@python_2_unicode_compatible
class Reference(models.Model):
    objects = ReferenceQuerySet.as_manager()

    name = models.CharField(
        verbose_name=_('Name'),
        max_length=50,
        blank=True,
    )
    image = models.ImageField(
        verbose_name=_('Icon'),
    )
    description = models.TextField(
        verbose_name=_('Description'),
        blank=True,
    )
    link = models.URLField(
        verbose_name=_('Link'),
        blank=True,
    )
    categories = models.ManyToManyField(
        Category,
        verbose_name=_('Categories'),
        related_name='references'
    )
    order = models.IntegerField(
        verbose_name=_('Order'),
        default=0,
    )
    enabled = models.BooleanField(
        verbose_name=_('Enabled'),
        default=True,
    )

    def __repr__(self):
        return '{}: {}'.format(self.id, self.name)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Reference')
        verbose_name_plural = _('References')
        ordering = [
            '-order',
        ]


@python_2_unicode_compatible
class ReferencePlugin(CMSPlugin):
    categories = models.ManyToManyField(
        Category,
        verbose_name=_('Categories'),
        related_name='plugins',
    )

    def __str__(self):
        categories = self.categories.all()
        if not categories:
            return ugettext('Empty')
        return ', '.join(force_text(c) for c in categories)

    class Meta:
        verbose_name = _('Reference Plugin')
        verbose_name_plural = _('Reference Plugins')

    def copy_relations(self, old_instance):
        self.categories = old_instance.categories.all()
